﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Threading;
using System.IO.Ports;
using Leap;


namespace LeapMotionSpeakerController
{
    class Program
    {
        //CLASS WIDE VARIABLES

        //serial port
        SerialPort sp = new SerialPort("COM14", 9600, Parity.None, 8, StopBits.One);
        string userInput = " ";
        string notesToPrint = " ";
        string mary = " 5  4  3  4  5  5  5  4  4  4  5  7  7  5  4  3  4  5  5  5  5  4  4  5  4  3 ";
        string other = " 7  7  1  7  3  2  7  7  1  7  4  3  7  7  7  5  3  2  1  6  6  5  3  4  3";

        //Leap Motion Variables
        Controller controller = new Controller();
        Frame frame = new Frame();

        //main
        static void Main(string[] args)
        {
            new Program();

        }//end main


//#Default constructor controlling the infinite loop of reading and sending data#/
        public Program() : base()
        {
            //recognize finger gesture
            controller.EnableGesture(Gesture.GestureType.TYPE_SCREEN_TAP);

            //open COM port if not already open
            if (!sp.IsOpen)
            {
                OpenConnection();
                sp.DiscardInBuffer();
            }
            Console.Write("Connection Successful!\n\n");
            goToMenu();
            Thread.Sleep(2000);

            //just get input from leap motion and send data forever
            while (true)
            {
                serialSender(getLeapMotion());
            }

        }//end default constructor

        public bool goToMenu()
        {
            Console.Write("**************************************************************\n");
            Console.Write("Welcome to the Luft Maestro!!!\n");
            Console.Write("What would you like to do??\n\n");
            Console.Write("1.                   Give me song notes!!\n");
            Console.Write("2.                   Go to manual mode   \n");
            Console.Write("3.                   Exit application   \n");
            Console.Write("**************************************************************\n\n");

            userInput = Console.ReadLine();
            Console.Write("You just input " + userInput + " \n");


            if(userInput == "1")
            {
                Console.Write("Entering note menu\n");
                notesToPrint = goToNoteMenu();

            }
            else if(userInput == "2")
            {

                Console.Write("Entering manual mode\n");

            }
            else
            {
                Console.Write("\n\nExiting application, good bye!! :) :) :)\n\n");
                System.Environment.Exit(1);
            }

            return true;
        }//goToMainMenu


        public string goToNoteMenu()
        {
            Console.Write("**************************************************************\n");
            Console.Write("What song would you like to play??\n\n");
            Console.Write("1.                   \" Mary Had a Little Lamb\"\n");
            Console.Write("2.                   \" Happy Birthday\" \n");
            Console.Write("3.                    Go to manual mode\n");
            Console.Write("**************************************************************\n\n");
            userInput = Console.ReadLine();

            if (userInput == "1")
                return mary;
            else if (userInput == "2")
                return other;
            else
                return " ";

        }//end goToNoteMenu


//#Get information from the leap motion device#/
        public int getLeapMotion()
        {
            int numberOfFingers;    

            frame = controller.Frame();             //get current frame and compute current fingers 
            numberOfFingers = frame.Fingers.Extended().Count;

            //Console.Write("Returning"+temp+"\n");
            return numberOfFingers;

        }//end getLeapMotion


//Send the data over the COM port/
        public void serialSender(int output)
        {
            char[] buffer = new char[1];        //buffer of 1 byte

            char numberOfFingers = Convert.ToChar( (output+96) );

            buffer[0] = numberOfFingers;                 //load output buffer

            if (sp.IsOpen)
            {
                sp.Write(buffer, 0, 1);             //send buffer, offset 0, 1 byte
            }

            Console.Clear();
            Console.Write("\n\n\n"+notesToPrint);
            Console.Write("\n\n\n\n\n\n\n\n\nsending "+output+"\n\n");
            Thread.Sleep(650);

        }//end serial sender


//Open COM port connection
        public void OpenConnection()
        {
            if (sp != null)
            {
                sp.Open();  // opens the connection
                sp.ReadTimeout = 25;  // sets the timeout value before reporting error (in ms)
                                        //sp.WriteTimeout = 100;  // sets the timeout value before reporting error (in ms)
                Console.Write("Port Opened!\n");
            }
            else
            {
                if (sp.IsOpen)
                {
                    Console.Write("Port is already open");
                }
                else
                {
                    Console.Write("Port == null");
                }
            }
        }//end openConnection


//Make sure to close the COM port before exiting/
        void OnApplicationQuit()
        {
            if (sp.IsOpen)
            {
                Console.Write("connection closed!");
                sp.Close();
            }

            controller.Dispose();

        }//end on app quit

    }//end program
}//end file